@@ -44,9 +44,11 @@ OVN_PORT_BINDING_PROFILE_PARAMS = [{'parent_name': six.string_types,
                                     'tag': six.integer_types},
                                    {'vtep-physical-switch': six.string_types,
                                     'vtep-logical-switch': six.string_types}]
+MIGRATING_ATTR = 'migrating_to'
 OVN_ROUTER_PORT_OPTION_KEYS = ['router-port', 'nat-addresses']
 OVN_GATEWAY_CHASSIS_KEY = 'redirect-chassis'
 OVN_GATEWAY_NAT_ADDRESSES_KEY = 'nat-addresses'
+OVN_DROP_PORT_GROUP_NAME = 'neutron_pg_drop'
 
 OVN_PROVNET_PORT_NAME_PREFIX = 'provnet-'
 
@@ -96,6 +98,10 @@ DB_MAX_RETRY_INTERVAL = 1
 TXN_COMMITTED = 'committed'
 INITIAL_REV_NUM = -1
 
+ACL_EXPECTED_COLUMNS_NBDB = (
+    'external_ids', 'direction', 'log', 'priority',
+    'name', 'action', 'severity', 'match')
+
 # Resource types
 TYPE_NETWORKS = 'networks'
 TYPE_PORTS = 'ports'
