@@ -53,7 +53,8 @@ OvnPortInfo = collections.namedtuple(
 
 
 GW_INFO = collections.namedtuple('GatewayInfo', ['network_id', 'subnet_id',
-                                                 'router_ip', 'gateway_ip'])
+                                                 'router_ip', 'gateway_ip',
+                                                 'ip_version', 'ip_prefix'])
 
 
 class OVNClient(object):
@@ -283,41 +284,72 @@ class OVNClient(object):
             # The lport_name *must* be neutron port['id'].  It must match the
             # iface-id set in the Interfaces table of the Open_vSwitch
             # database which nova sets to be the port ID.
-            txn.add(self._nb_idl.create_lswitch_port(
-                    lport_name=port['id'],
-                    lswitch_name=lswitch_name,
-                    addresses=port_info.addresses,
-                    external_ids=external_ids,
-                    parent_name=port_info.parent_name,
-                    tag=port_info.tag,
-                    enabled=port.get('admin_state_up'),
-                    options=port_info.options,
-                    type=port_info.type,
-                    port_security=port_info.port_security,
-                    dhcpv4_options=dhcpv4_options,
-                    dhcpv6_options=dhcpv6_options))
 
-            acls_new = ovn_acl.add_acls(self._plugin, admin_context,
-                                        port, sg_cache, subnet_cache,
-                                        self._nb_idl)
-            for acl in acls_new:
-                txn.add(self._nb_idl.add_acl(**acl))
+            kwargs = {
+                'lport_name': port['id'],
+                'lswitch_name': lswitch_name,
+                'addresses': port_info.addresses,
+                'external_ids': external_ids,
+                'parent_name': port_info.parent_name,
+                'tag': port_info.tag,
+                'enabled': port.get('admin_state_up'),
+                'options': port_info.options,
+                'type': port_info.type,
+                'port_security': port_info.port_security,
+                'dhcpv4_options': dhcpv4_options,
+                'dhcpv6_options': dhcpv6_options
+            }
 
+            # NOTE(mjozefcz): Do not set addresses if the port is not
+            # bound and has no device_owner - possibly it is a VirtualIP
+            # port used for Octavia (VRRP).
+            # For more details check related bug #1789686.
+            if (not port.get('device_owner') and
+                port.get(portbindings.VIF_TYPE) ==
+                    portbindings.VIF_TYPE_UNBOUND):
+                kwargs['addresses'] = []
+
+            port_cmd = txn.add(self._nb_idl.create_lswitch_port(**kwargs))
+
+            # Handle ACL's for this port. If we're not using Port Groups
+            # because either the schema doesn't support it or we didn't
+            # migrate old SGs from Address Sets to Port Groups, then we
+            # keep the old behavior. For those SGs this port belongs to
+            # that are modelled as a Port Group, we'll use it.
             sg_ids = utils.get_lsp_security_groups(port)
-            if port.get('fixed_ips') and sg_ids:
-                addresses = ovn_acl.acl_port_ips(port)
-                # NOTE(rtheis): Fail port creation if the address set doesn't
-                # exist. This prevents ports from being created on any security
-                # groups out-of-sync between neutron and OVN.
-                for sg_id in sg_ids:
-                    for ip_version in addresses:
-                        if addresses[ip_version]:
-                            txn.add(self._nb_idl.update_address_set(
-                                name=utils.ovn_addrset_name(sg_id,
-                                                            ip_version),
-                                addrs_add=addresses[ip_version],
-                                addrs_remove=None,
-                                if_exists=False))
+            if self._nb_idl.is_port_groups_supported():
+                # If this is not a trusted port or port security is enabled,
+                # add it to the default drop Port Group so that all traffic
+                # is dropped by default.
+                if not utils.is_lsp_trusted(port) or port_info.port_security:
+                    self._add_port_to_drop_port_group(port_cmd, txn)
+                # For SGs modelled as OVN Port Groups, just add the port to
+                # its Port Group.
+                for sg in sg_ids:
+                    txn.add(self._nb_idl.pg_add_ports(
+                        utils.ovn_port_group_name(sg), port_cmd))
+            else:
+                # SGs modelled as Address Sets:
+                acls_new = ovn_acl.add_acls(self._plugin, admin_context,
+                                            port, sg_cache, subnet_cache,
+                                            self._nb_idl)
+                for acl in acls_new:
+                    txn.add(self._nb_idl.add_acl(**acl))
+
+                if port.get('fixed_ips') and sg_ids:
+                    addresses = ovn_acl.acl_port_ips(port)
+                    # NOTE(rtheis): Fail port creation if the address set
+                    # doesn't exist. This prevents ports from being created on
+                    # any security groups out-of-sync between neutron and OVN.
+                    for sg_id in sg_ids:
+                        for ip_version in addresses:
+                            if addresses[ip_version]:
+                                txn.add(self._nb_idl.update_address_set(
+                                    name=utils.ovn_addrset_name(sg_id,
+                                                                ip_version),
+                                    addrs_add=addresses[ip_version],
+                                    addrs_remove=None,
+                                    if_exists=False))
 
             if self.is_dns_required_for_port(port):
                 self.add_txns_to_sync_port_dns_records(txn, port)
@@ -382,6 +414,16 @@ class OVNClient(object):
                 dhcpv6_options = txn.add(port_info.dhcpv6_options['cmd'])
             else:
                 dhcpv6_options = [port_info.dhcpv6_options['uuid']]
+
+            # NOTE(mjozefcz): Do not set addresses if the port is not
+            # bound and has no device_owner - possibly it is a VirtualIP
+            # port used for Octavia (VRRP).
+            # For more details check related bug #1789686.
+            if (not port.get('device_owner') and
+                port.get(portbindings.VIF_TYPE) ==
+                    portbindings.VIF_TYPE_UNBOUND):
+                columns_dict['addresses'] = []
+
             # NOTE(lizk): Fail port updating if port doesn't exist. This
             # prevents any new inserted resources to be orphan, such as port
             # dhcp options or ACL rules for port, e.g. a port was created
@@ -424,63 +466,86 @@ class OVNClient(object):
             port_security_changed = utils.is_port_security_enabled(port) != (
                 bool(ovn_port.port_security))
 
-            # Refresh ACLs for changed security groups or fixed IPs.
-            if detached_sg_ids or attached_sg_ids or is_fixed_ips_updated or (
-                    port_security_changed):
-                # Note that update_acls will compare the port's ACLs to
-                # ensure only the necessary ACLs are added and deleted
-                # on the transaction.
-                acls_new = ovn_acl.add_acls(self._plugin,
-                                            admin_context,
-                                            port,
-                                            sg_cache,
-                                            subnet_cache,
-                                            self._nb_idl)
-                txn.add(self._nb_idl.update_acls([port['network_id']],
-                                                 [port],
-                                                 {port['id']: acls_new},
-                                                 need_compare=True))
+            if self._nb_idl.is_port_groups_supported():
+                for sg in detached_sg_ids:
+                    txn.add(self._nb_idl.pg_del_ports(
+                        utils.ovn_port_group_name(sg), port['id']))
+                for sg in attached_sg_ids:
+                    txn.add(self._nb_idl.pg_add_ports(
+                        utils.ovn_port_group_name(sg), port['id']))
+                if (not utils.is_lsp_trusted(port) and
+                        utils.is_port_security_enabled(port)):
+                    self._add_port_to_drop_port_group(port['id'], txn)
+                # If the port doesn't belong to any security group and
+                # port_security is disabled, or it's a trusted port, then
+                # allow all traffic.
+                elif ((not new_sg_ids and
+                      not utils.is_port_security_enabled(port)) or
+                      utils.is_lsp_trusted(port)):
+                    self._del_port_from_drop_port_group(port['id'], txn)
+            else:
+                # Refresh ACLs for changed security groups or fixed IPs.
+                if (detached_sg_ids or attached_sg_ids or
+                        is_fixed_ips_updated or port_security_changed):
+                    # Note that update_acls will compare the port's ACLs to
+                    # ensure only the necessary ACLs are added and deleted
+                    # on the transaction.
+                    acls_new = ovn_acl.add_acls(self._plugin,
+                                                admin_context,
+                                                port,
+                                                sg_cache,
+                                                subnet_cache,
+                                                self._nb_idl)
+                    txn.add(self._nb_idl.update_acls([port['network_id']],
+                                                     [port],
+                                                     {port['id']: acls_new},
+                                                     need_compare=True))
 
-            # Refresh address sets for changed security groups or fixed IPs.
-            if len(old_fixed_ips) != 0 or len(new_fixed_ips) != 0:
-                addresses = ovn_acl.acl_port_ips(port)
-                addresses_old = utils.sort_ips_by_version(
-                    utils.get_ovn_port_addresses(ovn_port))
-                # Add current addresses to attached security groups.
-                for sg_id in attached_sg_ids:
-                    for ip_version in addresses:
-                        if addresses[ip_version]:
-                            txn.add(self._nb_idl.update_address_set(
-                                name=utils.ovn_addrset_name(sg_id, ip_version),
-                                addrs_add=addresses[ip_version],
-                                addrs_remove=None))
-                # Remove old addresses from detached security groups.
-                for sg_id in detached_sg_ids:
-                    for ip_version in addresses_old:
-                        if addresses_old[ip_version]:
-                            txn.add(self._nb_idl.update_address_set(
-                                name=utils.ovn_addrset_name(sg_id, ip_version),
-                                addrs_add=None,
-                                addrs_remove=addresses_old[ip_version]))
-
-                if is_fixed_ips_updated or is_allowed_ips_updated:
-                    # We have refreshed address sets for attached and detached
-                    # security groups, so now we only need to take care of
-                    # unchanged security groups.
-                    unchanged_sg_ids = new_sg_ids & old_sg_ids
-                    for sg_id in unchanged_sg_ids:
+                # Refresh address sets for changed security groups or fixed
+                # IPs.
+                if len(old_fixed_ips) != 0 or len(new_fixed_ips) != 0:
+                    addresses = ovn_acl.acl_port_ips(port)
+                    addresses_old = utils.sort_ips_by_version(
+                        utils.get_ovn_port_addresses(ovn_port))
+                    # Add current addresses to attached security groups.
+                    for sg_id in attached_sg_ids:
                         for ip_version in addresses:
-                            addr_add = (set(addresses[ip_version]) -
-                                        set(addresses_old[ip_version])) or None
-                            addr_remove = (set(addresses_old[ip_version]) -
-                                           set(addresses[ip_version])) or None
-
-                            if addr_add or addr_remove:
+                            if addresses[ip_version]:
                                 txn.add(self._nb_idl.update_address_set(
-                                        name=utils.ovn_addrset_name(
-                                            sg_id, ip_version),
-                                        addrs_add=addr_add,
-                                        addrs_remove=addr_remove))
+                                    name=utils.ovn_addrset_name(sg_id,
+                                        ip_version),
+                                    addrs_add=addresses[ip_version],
+                                    addrs_remove=None))
+                    # Remove old addresses from detached security groups.
+                    for sg_id in detached_sg_ids:
+                        for ip_version in addresses_old:
+                            if addresses_old[ip_version]:
+                                txn.add(self._nb_idl.update_address_set(
+                                    name=utils.ovn_addrset_name(sg_id,
+                                        ip_version),
+                                    addrs_add=None,
+                                    addrs_remove=addresses_old[ip_version]))
+
+                    if is_fixed_ips_updated or is_allowed_ips_updated:
+                        # We have refreshed address sets for attached and
+                        # detached security groups, so now we only need to take
+                        # care of unchanged security groups.
+                        unchanged_sg_ids = new_sg_ids & old_sg_ids
+                        for sg_id in unchanged_sg_ids:
+                            for ip_version in addresses:
+                                addr_add = ((set(addresses[ip_version]) -
+                                             set(addresses_old[ip_version])) or
+                                            None)
+                                addr_remove = (
+                                    (set(addresses_old[ip_version]) -
+                                     set(addresses[ip_version])) or None)
+
+                                if addr_add or addr_remove:
+                                    txn.add(self._nb_idl.update_address_set(
+                                            name=utils.ovn_addrset_name(
+                                                sg_id, ip_version),
+                                            addrs_add=addr_add,
+                                            addrs_remove=addr_remove))
 
             if self.is_dns_required_for_port(port):
                 self.add_txns_to_sync_port_dns_records(
@@ -505,20 +570,22 @@ class OVNClient(object):
         with self._nb_idl.transaction(check_error=True) as txn:
             txn.add(self._nb_idl.delete_lswitch_port(
                 port_id, network_id))
-            txn.add(self._nb_idl.delete_acl(network_id, port_id))
 
-            addresses = utils.sort_ips_by_version(
-                utils.get_ovn_port_addresses(ovn_port))
-            sec_groups = self._get_lsp_backward_compat_sgs(
-                ovn_port, port_object=port_object, skip_trusted_port=False)
-            for sg_id in sec_groups:
-                for ip_version, addr_list in addresses.items():
-                    if not addr_list:
-                        continue
-                    txn.add(self._nb_idl.update_address_set(
-                        name=utils.ovn_addrset_name(sg_id, ip_version),
-                        addrs_add=None,
-                        addrs_remove=addr_list))
+            if not self._nb_idl.is_port_groups_supported():
+                txn.add(self._nb_idl.delete_acl(network_id, port_id))
+
+                addresses = utils.sort_ips_by_version(
+                    utils.get_ovn_port_addresses(ovn_port))
+                sec_groups = self._get_lsp_backward_compat_sgs(
+                    ovn_port, port_object=port_object, skip_trusted_port=False)
+                for sg_id in sec_groups:
+                    for ip_version, addr_list in addresses.items():
+                        if not addr_list:
+                            continue
+                        txn.add(self._nb_idl.update_address_set(
+                            name=utils.ovn_addrset_name(sg_id, ip_version),
+                            addrs_add=None,
+                            addrs_remove=addr_list))
 
             if port_object and self.is_dns_required_for_port(port_object):
                 self.add_txns_to_remove_port_dns_records(txn, port_object)
@@ -541,9 +608,7 @@ class OVNClient(object):
         context = n_context.get_admin_context()
         fip_db = self._l3_plugin._get_floatingip(context, floatingip['id'])
 
-        func = self._nb_idl.add_nat_rule_in_lrouter
         gw_lrouter_name = utils.ovn_name(router_id)
-        nat_rule_args = (gw_lrouter_name,)
         # TODO(chandrav): Since the floating ip port is not
         # bound to any chassis, packets destined to floating ip
         # will be dropped. To overcome this, delete the floating
@@ -554,21 +619,6 @@ class OVNClient(object):
                         fip_db['floating_port_id'],
                         utils.ovn_name(floatingip['floating_network_id'])))
 
-        # Get the list of nat rules and check if the external_ip
-        # with type 'dnat_and_snat' already exists or not.
-        # If exists, set the new value.
-        # This happens when the port associated to a floating ip
-        # is deleted before the disassociation.
-        lrouter_nat_rules = self._nb_idl.get_lrouter_nat_rules(
-            gw_lrouter_name)
-        for nat_rule in lrouter_nat_rules:
-            if (nat_rule['external_ip'] ==
-                    floatingip['floating_ip_address'] and
-                    nat_rule['type'] == 'dnat_and_snat'):
-                func = self._nb_idl.set_nat_rule_in_lrouter
-                nat_rule_args = (gw_lrouter_name, nat_rule['uuid'])
-                break
-
         ext_ids = {
             ovn_const.OVN_FIP_EXT_ID_KEY: floatingip['id'],
             ovn_const.OVN_REV_NUM_EXT_ID_KEY: str(utils.get_revision_number(
@@ -585,14 +635,19 @@ class OVNClient(object):
             columns['logical_port'] = floatingip['port_id']
             ext_ids[ovn_const.OVN_FIP_EXT_MAC_KEY] = port['mac_address']
             if self._nb_idl.lsp_get_up(floatingip['port_id']).execute():
-                columns['external_mac'] = port['mac_address']
+                port_net = self._plugin.get_network(context,
+                                                    port['network_id'])
+                if port_net.get(pnet.NETWORK_TYPE) != const.TYPE_VLAN:
+                    columns['external_mac'] = port['mac_address']
 
         # TODO(dalvarez): remove this check once the minimum OVS required
         # version contains the column (when OVS 2.8.2 is released).
         if self._nb_idl.is_col_present('NAT', 'external_ids'):
             columns['external_ids'] = ext_ids
 
-        commands.append(func(*nat_rule_args, **columns))
+        commands.append(self._nb_idl.add_nat_rule_in_lrouter(gw_lrouter_name,
+                                                             **columns))
+
         self._transaction(commands, txn=txn)
 
     def _delete_floatingip(self, fip, lrouter, txn=None):
@@ -709,16 +764,18 @@ class OVNClient(object):
                           'router. Error: %s', e)
 
     def _get_gw_info(self, context, router):
+        gateways_info = []
         ext_gw_info = router.get(l3.EXTERNAL_GW_INFO, {})
         network_id = ext_gw_info.get('network_id', '')
         for ext_fixed_ip in ext_gw_info.get('external_fixed_ips', []):
             subnet_id = ext_fixed_ip['subnet_id']
             subnet = self._plugin.get_subnet(context, subnet_id)
-            if subnet['ip_version'] == 4:
-                return GW_INFO(
-                    network_id, subnet_id, ext_fixed_ip['ip_address'],
-                    subnet.get('gateway_ip'))
-        return GW_INFO('', '', '', '')
+            gateways_info.append(GW_INFO(
+                network_id, subnet_id, ext_fixed_ip['ip_address'],
+                subnet.get('gateway_ip'), subnet['ip_version'],
+                const.IPv4_ANY if subnet['ip_version'] == const.IP_VERSION_4
+                else const.IPv6_ANY))
+        return gateways_info
 
     def _delete_router_ext_gw(self, context, router, networks, txn):
         if not networks:
@@ -726,17 +783,19 @@ class OVNClient(object):
         router_id = router['id']
         gw_port_id = router['gw_port_id']
         gw_lrouter_name = utils.ovn_name(router_id)
-        gw_info = self._get_gw_info(context, router)
-        txn.add(self._nb_idl.delete_static_route(gw_lrouter_name,
-                                                 ip_prefix='0.0.0.0/0',
-                                                 nexthop=gw_info.gateway_ip))
+        gateways = self._get_gw_info(context, router)
+        for gw_info in gateways:
+            if gw_info.ip_version == const.IP_VERSION_4:
+                for network in networks:
+                    txn.add(self._nb_idl.delete_nat_rule_in_lrouter(
+                        gw_lrouter_name, type='snat', logical_ip=network,
+                        external_ip=gw_info.router_ip))
+            txn.add(self._nb_idl.delete_static_route(
+                gw_lrouter_name, ip_prefix=gw_info.ip_prefix,
+                nexthop=gw_info.gateway_ip))
         txn.add(self._nb_idl.delete_lrouter_port(
             utils.ovn_lrouter_port_name(gw_port_id),
             gw_lrouter_name))
-        for network in networks:
-            txn.add(self._nb_idl.delete_nat_rule_in_lrouter(
-                gw_lrouter_name, type='snat', logical_ip=network,
-                external_ip=gw_info.router_ip))
 
     def _get_nets_and_ipv6_ra_confs_for_router_port(
             self, port_fixed_ips):
@@ -767,45 +826,55 @@ class OVNClient(object):
     def _add_router_ext_gw(self, context, router, networks, txn):
         router_id = router['id']
         # 1. Add the external gateway router port.
-        gw_info = self._get_gw_info(context, router)
+        gateways = self._get_gw_info(context, router)
         gw_port_id = router['gw_port_id']
         port = self._plugin.get_port(context, gw_port_id)
         self._create_lrouter_port(router_id, port, txn=txn)
 
-        # TODO(lucasagomes): Remove this check after OVS 2.8.2 is tagged
-        # (prior to that, the external_ids column didn't exist in this
-        # table).
-        columns = {}
-        if self._nb_idl.is_col_present('Logical_Router_Static_Route',
-                                       'external_ids'):
-            columns['external_ids'] = {
-                ovn_const.OVN_ROUTER_IS_EXT_GW: 'true',
-                ovn_const.OVN_SUBNET_EXT_ID_KEY: gw_info.subnet_id}
+        def _build_extids(gw_info):
+            # TODO(lucasagomes): Remove this check after OVS 2.8.2 is tagged
+            # (prior to that, the external_ids column didn't exist in this
+            # table).
+            columns = {}
+            if self._nb_idl.is_col_present('Logical_Router_Static_Route',
+                                           'external_ids'):
+                columns['external_ids'] = {
+                    ovn_const.OVN_ROUTER_IS_EXT_GW: 'true',
+                    ovn_const.OVN_SUBNET_EXT_ID_KEY: gw_info.subnet_id}
+            return columns
 
         # 2. Add default route with nexthop as gateway ip
         lrouter_name = utils.ovn_name(router_id)
-        txn.add(self._nb_idl.add_static_route(
-            lrouter_name, ip_prefix='0.0.0.0/0', nexthop=gw_info.gateway_ip,
-            **columns))
+        for gw_info in gateways:
+            columns = _build_extids(gw_info)
+            txn.add(self._nb_idl.add_static_route(
+                lrouter_name, ip_prefix=gw_info.ip_prefix,
+                nexthop=gw_info.gateway_ip, **columns))
 
         # 3. Add snat rules for tenant networks in lrouter if snat is enabled
         if utils.is_snat_enabled(router) and networks:
             self.update_nat_rules(router, networks, enable_snat=True, txn=txn)
         return port
 
-    def _check_external_ips_changed(self, context, ovn_snats, ovn_static_route,
-                                    router):
-        gw_info = self._get_gw_info(context, router)
+    def _check_external_ips_changed(self, context, ovn_snats,
+                                    ovn_static_routes, router):
+        gateways = self._get_gw_info(context, router)
+        ovn_gw_subnets = None
         if self._nb_idl.is_col_present('Logical_Router_Static_Route',
                                        'external_ids'):
-            ext_ids = getattr(ovn_static_route, 'external_ids', {})
-            if (ext_ids.get(ovn_const.OVN_SUBNET_EXT_ID_KEY) !=
-               gw_info.subnet_id):
-                return True
+            ovn_gw_subnets = [
+                getattr(route, 'external_ids', {}).get(
+                    ovn_const.OVN_SUBNET_EXT_ID_KEY) for route in
+                ovn_static_routes]
 
-        for snat in ovn_snats:
-            if snat.external_ip != gw_info.router_ip:
+        for gw_info in gateways:
+            if ovn_gw_subnets and gw_info.subnet_id not in ovn_gw_subnets:
                 return True
+            if gw_info.ip_version == 6:
+                continue
+            for snat in ovn_snats:
+                if snat.external_ip != gw_info.router_ip:
+                    return True
 
         return False
 
@@ -1249,12 +1318,12 @@ class OVNClient(object):
         func = (self._nb_idl.add_nat_rule_in_lrouter if enable_snat else
                 self._nb_idl.delete_nat_rule_in_lrouter)
         gw_lrouter_name = utils.ovn_name(router['id'])
-        gw_info = self._get_gw_info(context, router)
-        commands = []
-        for network in networks:
-            commands.append(
-                func(gw_lrouter_name, type='snat', logical_ip=network,
-                     external_ip=gw_info.router_ip))
+        gateways = self._get_gw_info(context, router)
+        # Update NAT rules only for IPv4 subnets
+        commands = [func(gw_lrouter_name, type='snat', logical_ip=network,
+                         external_ip=gw_info.router_ip) for gw_info in gateways
+                    if gw_info.ip_version != const.IP_VERSION_6
+                    for network in networks]
         self._transaction(commands, txn=txn)
 
     def _create_provnet_port(self, txn, network, physnet, tag):
@@ -1331,6 +1400,34 @@ class OVNClient(object):
         qos_update_required = self._is_qos_update_required(network)
         check_rev_cmd = self._nb_idl.check_revision_number(
             lswitch_name, network, ovn_const.TYPE_NETWORKS)
+
+        # TODO(numans) - When a network's dns domain name is updated, we need
+        # to update the DNS records for this network in DNS OVN NB DB table.
+        # (https://bugs.launchpad.net/networking-ovn/+bug/1777978)
+        # Eg. if the network n1's dns domain name was "test1" and if it has
+        # 2 bound ports - p1 and p2, we would have created the below dns
+        # records
+        # ===========================
+        # p1 = P1_IP
+        # p1.test1 = P1_IP
+        # p1.default_domain = P1_IP
+        # p2 = P2_IP
+        # p2.test1 = P2_IP
+        # p2.default_domain = P2_IP
+        # ===========================
+        # if the network n1's dns domain name is updated to test2, then we need
+        # to delete the below DNS records
+        # ===========================
+        # p1.test1 = P1_IP
+        # p2.test1 = P2_IP
+        # ===========================
+        # and add the new ones
+        # ===========================
+        # p1.test2 = P1_IP
+        # p2.test2 = P2_IP
+        # ===========================
+        # in the DNS row for this network.
+
         with self._nb_idl.transaction(check_error=True) as txn:
             txn.add(check_rev_cmd)
             ext_ids = self._gen_network_external_ids(network)
@@ -1552,7 +1649,6 @@ class OVNClient(object):
             if subnet['ip_version'] == 4:
                 context = n_context.get_admin_context()
                 self.update_metadata_port(context, network['id'])
-
             self._add_subnet_dhcp_options(subnet, network)
         db_rev.bump_revision(subnet, ovn_const.TYPE_SUBNETS)
 
@@ -1584,19 +1680,64 @@ class OVNClient(object):
         db_rev.delete_revision(subnet_id, ovn_const.TYPE_SUBNETS)
 
     def create_security_group(self, security_group):
+        # If the OVN schema supports Port Groups, we'll model security groups
+        # as such. Otherwise, for backwards compatibility, we'll keep creating
+        # two Address Sets for each Neutron SG (one for IPv4 and one for
+        # IPv6).
         with self._nb_idl.transaction(check_error=True) as txn:
-            for ip_version in ('ip4', 'ip6'):
-                name = utils.ovn_addrset_name(security_group['id'], ip_version)
-                ext_ids = {ovn_const.OVN_SG_EXT_ID_KEY: security_group['id']}
-                txn.add(self._nb_idl.create_address_set(
-                    name=name, external_ids=ext_ids))
+            ext_ids = {ovn_const.OVN_SG_EXT_ID_KEY: security_group['id']}
+            if self._nb_idl.is_port_groups_supported():
+                name = utils.ovn_port_group_name(security_group['id'])
+                txn.add(self._nb_idl.pg_add(
+                    name=name, acls=[], external_ids=ext_ids))
+                # When a SG is created, it comes with some default rules,
+                # so we'll apply them to the Port Group.
+                ovn_acl.add_acls_for_sg_port_group(self._nb_idl,
+                                                   security_group, txn)
+            else:
+                for ip_version in ('ip4', 'ip6'):
+                    name = utils.ovn_addrset_name(security_group['id'],
+                                                  ip_version)
+                    txn.add(self._nb_idl.create_address_set(
+                        name=name, external_ids=ext_ids))
         db_rev.bump_revision(security_group, ovn_const.TYPE_SECURITY_GROUPS)
 
+    def create_default_drop_port_group(self, ports=None):
+        pg_name = ovn_const.OVN_DROP_PORT_GROUP_NAME
+        with self._nb_idl.transaction(check_error=True) as txn:
+            if not self._nb_idl.get_port_group(pg_name):
+                # If drop Port Group doesn't exist yet, create it.
+                txn.add(self._nb_idl.pg_add(pg_name, acls=[], may_exist=True))
+                # Add ACLs to this Port Group so that all traffic is dropped.
+                acls = ovn_acl.add_acls_for_drop_port_group(pg_name)
+                for acl in acls:
+                    txn.add(self._nb_idl.pg_acl_add(may_exist=True, **acl))
+
+            if ports:
+                ports_ids = [port['id'] for port in ports]
+                # Add the ports to the default Port Group
+                txn.add(self._nb_idl.pg_add_ports(pg_name, ports_ids))
+
+    def _add_port_to_drop_port_group(self, port, txn):
+        self.create_default_drop_port_group()
+        txn.add(self._nb_idl.pg_add_ports(ovn_const.OVN_DROP_PORT_GROUP_NAME,
+                port))
+
+    def _del_port_from_drop_port_group(self, port, txn):
+        pg_name = ovn_const.OVN_DROP_PORT_GROUP_NAME
+        if self._nb_idl.get_port_group(pg_name):
+            txn.add(self._nb_idl.pg_del_ports(pg_name, port))
+
     def delete_security_group(self, security_group_id):
         with self._nb_idl.transaction(check_error=True) as txn:
-            for ip_version in ('ip4', 'ip6'):
-                name = utils.ovn_addrset_name(security_group_id, ip_version)
-                txn.add(self._nb_idl.delete_address_set(name=name))
+            if self._nb_idl.is_port_groups_supported():
+                name = utils.ovn_port_group_name(security_group_id)
+                txn.add(self._nb_idl.pg_del(name=name))
+            else:
+                for ip_version in ('ip4', 'ip6'):
+                    name = utils.ovn_addrset_name(security_group_id,
+                                                  ip_version)
+                    txn.add(self._nb_idl.delete_address_set(name=name))
         db_rev.delete_revision(security_group_id,
                                ovn_const.TYPE_SECURITY_GROUPS)
 
@@ -1714,14 +1855,24 @@ class OVNClient(object):
 
     def get_port_dns_records(self, port):
         port_dns_records = {}
+        net = port.get('network', {})
+        net_dns_domain = net.get('dns_domain', '').rstrip('.')
+
         for dns_assignment in port.get('dns_assignment', []):
             hostname = dns_assignment['hostname']
             fqdn = dns_assignment['fqdn'].rstrip('.')
+            net_dns_fqdn = hostname + '.' + net_dns_domain
             if hostname not in port_dns_records:
                 port_dns_records[hostname] = dns_assignment['ip_address']
+                if net_dns_domain and net_dns_fqdn != fqdn:
+                    port_dns_records[net_dns_fqdn] = (
+                        dns_assignment['ip_address'])
             else:
                 port_dns_records[hostname] += " " + (
                     dns_assignment['ip_address'])
+                if net_dns_domain and net_dns_fqdn != fqdn:
+                    port_dns_records[hostname + '.' + net_dns_domain] += (
+                        " " + dns_assignment['ip_address'])
 
             if fqdn not in port_dns_records:
                 port_dns_records[fqdn] = dns_assignment['ip_address']
@@ -1771,11 +1922,19 @@ class OVNClient(object):
         if ls_dns_record is None:
             return
 
+        net = port.get('network', {})
+        net_dns_domain = net.get('dns_domain', '').rstrip('.')
+
         hostnames = []
         for dns_assignment in port['dns_assignment']:
-            if dns_assignment['hostname'] not in hostnames:
-                hostnames.append(dns_assignment['hostname'])
+            hostname = dns_assignment['hostname']
             fqdn = dns_assignment['fqdn'].rstrip('.')
+            if hostname not in hostnames:
+                hostnames.append(hostname)
+                net_dns_fqdn = hostname + '.' + net_dns_domain
+                if net_dns_domain and net_dns_fqdn != fqdn:
+                    hostnames.append(net_dns_fqdn)
+
             if fqdn not in hostnames:
                 hostnames.append(fqdn)
 
