@@ -80,6 +80,21 @@ def ovn_addrset_name(sg_id, ip_version):
     return ('as-%s-%s' % (ip_version, sg_id)).replace('-', '_')
 
 
+def ovn_pg_addrset_name(sg_id, ip_version):
+    # The name of the address set for the given security group id modelled as a
+    # Port Group and ip version. The format is:
+    #   pg-<security group uuid>-<ip version>
+    # with all '-' replaced with '_'. This replacement is necessary
+    # because OVN doesn't support '-' in an address set name.
+    return ('pg-%s-%s' % (sg_id, ip_version)).replace('-', '_')
+
+
+def ovn_port_group_name(sg_id):
+    # The name of the port group for the given security group id.
+    # The format is: pg-<security group uuid>.
+    return ('pg-%s' % sg_id).replace('-', '_')
+
+
 def is_network_device_port(port):
     return port.get('device_owner', '').startswith(
         const.DEVICE_OWNER_PREFIXES)
@@ -296,11 +311,10 @@ def get_lrouter_ext_gw_static_route(ovn_router):
     # TODO(lucasagomes): Remove the try...except block after OVS 2.8.2
     # is tagged.
     try:
-        for route in getattr(ovn_router, 'static_routes', []):
-            external_ids = getattr(route, 'external_ids', {})
-            if strutils.bool_from_string(
-                external_ids.get(constants.OVN_ROUTER_IS_EXT_GW, 'false')):
-                return route
+        return [route for route in getattr(ovn_router, 'static_routes', []) if
+                strutils.bool_from_string(getattr(
+                    route, 'external_ids', {}).get(
+                        constants.OVN_ROUTER_IS_EXT_GW, 'false'))]
     except KeyError:
         pass
 
