@@ -12,6 +12,7 @@
 #    under the License.
 #
 
+import copy
 import threading
 
 from neutron_lib.api.definitions import portbindings
@@ -30,6 +31,7 @@ from oslo_log import log
 
 from neutron.common import utils as n_utils
 from neutron.db import provisioning_blocks
+from neutron.plugins.ml2 import db as ml2_db
 from neutron.services.segments import db as segment_service_db
 
 from networking_ovn._i18n import _
@@ -131,7 +133,7 @@ class OVNMechanismDriver(api.MechanismDriver):
             portbindings.VIF_TYPE_VHOST_USER: {
                 portbindings.CAP_PORT_FILTER: False,
                 portbindings.VHOST_USER_MODE:
-                portbindings.VHOST_USER_MODE_CLIENT,
+                portbindings.VHOST_USER_MODE_SERVER,
                 portbindings.VHOST_USER_OVS_PLUG: True
             }
         }
@@ -381,7 +383,8 @@ class OVNMechanismDriver(api.MechanismDriver):
             return
         utils.validate_and_get_data_from_binding_profile(port)
         if self._is_port_provisioning_required(port, context.host):
-            self._insert_port_provisioning_block(context._plugin_context, port)
+            self._insert_port_provisioning_block(context._plugin_context,
+                                                 port['id'])
 
         db_rev.create_initial_revision(port['id'],
                                        ovn_const.TYPE_PORTS,
@@ -425,13 +428,12 @@ class OVNMechanismDriver(api.MechanismDriver):
 
         return True
 
-    def _insert_port_provisioning_block(self, context, port):
+    def _insert_port_provisioning_block(self, context, port_id):
         # Insert a provisioning block to prevent the port from
         # transitioning to active until OVN reports back that
         # the port is up.
         provisioning_blocks.add_provisioning_component(
-            context,
-            port['id'], resources.PORT,
+            context, port_id, resources.PORT,
             provisioning_blocks.L2_AGENT_ENTITY
         )
 
@@ -470,7 +472,8 @@ class OVNMechanismDriver(api.MechanismDriver):
         drastically affect performance.  Raising an exception will
         result in the deletion of the resource.
         """
-        port = context.current
+        port = copy.deepcopy(context.current)
+        port['network'] = context.network.current
         self._ovn_client.create_port(port)
         self._notify_dhcp_updated(port['id'])
 
@@ -495,7 +498,8 @@ class OVNMechanismDriver(api.MechanismDriver):
         utils.validate_and_get_data_from_binding_profile(port)
         if self._is_port_provisioning_required(port, context.host,
                                                context.original_host):
-            self._insert_port_provisioning_block(context._plugin_context, port)
+            self._insert_port_provisioning_block(context._plugin_context,
+                                                 port['id'])
 
         if utils.is_lsp_router_port(port):
             # handle the case when an existing port is added to a
@@ -521,8 +525,30 @@ class OVNMechanismDriver(api.MechanismDriver):
         state. It is up to the mechanism driver to ignore state or
         state changes that it does not know or care about.
         """
-        port = context.current
-        original_port = context.original
+        port = copy.deepcopy(context.current)
+        port['network'] = context.network.current
+        original_port = copy.deepcopy(context.original)
+        original_port['network'] = context.network.current
+
+        # NOTE(mjozefcz): Check if port is in migration state. If so update
+        # the port status from DOWN to UP in order to generate 'fake'
+        # vif-interface-plugged event. This workaround is needed to
+        # perform live-migration with live_migration_wait_for_vif_plug=True.
+        if ((port['status'] == const.PORT_STATUS_DOWN and
+             ovn_const.MIGRATING_ATTR in port[portbindings.PROFILE].keys() and
+             port[portbindings.VIF_TYPE] == portbindings.VIF_TYPE_OVS)):
+            admin_context = n_context.get_admin_context()
+            LOG.info("Setting port %s status from DOWN to UP in order "
+                     "to emit vif-interface-plugged event.",
+                     port['id'])
+            self._plugin.update_port_status(admin_context, port['id'],
+                                            const.PORT_STATUS_ACTIVE)
+            # The revision has been changed. In the meantime
+            # port-update event already updated the OVN configuration,
+            # So there is no need to update it again here. Anyway it
+            # will fail that OVN has port with bigger revision.
+            return
+
         self._ovn_client.update_port(port, port_object=original_port)
         self._notify_dhcp_updated(port['id'])
 
@@ -538,7 +564,8 @@ class OVNMechanismDriver(api.MechanismDriver):
         expected, and will not prevent the resource from being
         deleted.
         """
-        port = context.current
+        port = copy.deepcopy(context.current)
+        port['network'] = context.network.current
         self._ovn_client.delete_port(port['id'], port_object=port)
 
     def bind_port(self, context):
@@ -723,7 +750,7 @@ class OVNMechanismDriver(api.MechanismDriver):
                 'NAT', nat['_uuid'],
                 ('external_mac', mac)).execute(check_error=True)
         else:
-            LOG.debug("Clearing up external_mac of port %s", port_id, mac)
+            LOG.debug("Clearing up external_mac of port %s", port_id)
             self._nb_ovn.db_clear(
                 'NAT', nat['_uuid'], 'external_mac').execute(check_error=True)
 
@@ -755,12 +782,18 @@ class OVNMechanismDriver(api.MechanismDriver):
             # becasue the router ports are unbind so, for OVN we are
             # forcing the status here. Maybe it's something that we can
             # change in core Neutron in the future.
-            port = self._plugin.get_port(admin_context, port_id)
-            if port.get('device_owner') in (const.DEVICE_OWNER_ROUTER_INTF,
-                                            const.DEVICE_OWNER_DVR_INTERFACE,
-                                            const.DEVICE_OWNER_ROUTER_HA_INTF):
+            db_port = ml2_db.get_port(admin_context, port_id)
+            if not db_port:
+                return
+
+            if db_port.device_owner in (const.DEVICE_OWNER_ROUTER_INTF,
+                                        const.DEVICE_OWNER_DVR_INTERFACE,
+                                        const.DEVICE_OWNER_ROUTER_HA_INTF):
                 self._plugin.update_port_status(admin_context, port_id,
                                                 const.PORT_STATUS_ACTIVE)
+            elif db_port.device_owner.startswith(
+                    const.DEVICE_OWNER_COMPUTE_PREFIX):
+                self._plugin.nova_notifier.notify_port_active_direct(db_port)
         except (os_db_exc.DBReferenceError, n_exc.PortNotFound):
             LOG.debug('Port not found during OVN status up report: %s',
                       port_id)
@@ -775,11 +808,21 @@ class OVNMechanismDriver(api.MechanismDriver):
         self._update_dnat_entry_if_needed(port_id, False)
         admin_context = n_context.get_admin_context()
         try:
-            port = self._plugin.get_port(admin_context, port_id)
-            self._insert_port_provisioning_block(admin_context, port)
-            self._plugin.update_port_status(admin_context,
-                                            port['id'],
+            db_port = ml2_db.get_port(admin_context, port_id)
+            if not db_port:
+                return
+
+            self._insert_port_provisioning_block(admin_context, port_id)
+            self._plugin.update_port_status(admin_context, port_id,
                                             const.PORT_STATUS_DOWN)
+
+            if db_port.device_owner.startswith(
+                    const.DEVICE_OWNER_COMPUTE_PREFIX):
+                self._plugin.nova_notifier.record_port_status_changed(
+                    db_port, const.PORT_STATUS_ACTIVE, const.PORT_STATUS_DOWN,
+                    None)
+                self._plugin.nova_notifier.send_port_status(
+                    None, None, db_port)
         except (os_db_exc.DBReferenceError, n_exc.PortNotFound):
             LOG.debug("Port not found during OVN status down report: %s",
                       port_id)
@@ -815,8 +858,8 @@ class OVNMechanismDriver(api.MechanismDriver):
         """Wait for metadata service to be provisioned.
 
         Wait until metadata service has been setup for this port in the chassis
-        it resides. If metadata is disabled, this function will return right
-        away.
+        it resides. If metadata is disabled or DHCP is not enabled for its
+        subnets, this function will return right away.
         """
         if config.is_ovn_metadata_enabled() and self._sb_ovn:
             # Wait until metadata service has been setup for this port in the
@@ -831,6 +874,26 @@ class OVNMechanismDriver(api.MechanismDriver):
                 LOG.warning("Logical port %s is not bound to a "
                             "chassis", port_id)
                 return
+
+            # Check if the port belongs to some IPv4 subnet with DHCP enabled.
+            context = n_context.get_admin_context()
+            port = self._plugin.get_port(context, port_id)
+            port_subnet_ids = set(
+                ip['subnet_id'] for ip in port['fixed_ips'] if
+                n_utils.get_ip_version(ip['ip_address']) == const.IP_VERSION_4)
+            if not port_subnet_ids:
+                # The port doesn't belong to any IPv4 subnet
+                return
+
+            subnets = self._plugin.get_subnets(context, filters=dict(
+                network_id=[port['network_id']], ip_version=[4],
+                enable_dhcp=True))
+
+            subnet_ids = set(
+                s['id'] for s in subnets if s['id'] in port_subnet_ids)
+            if not subnet_ids:
+                return
+
             try:
                 n_utils.wait_until_true(
                     lambda: datapath in
