@@ -15,11 +15,11 @@ from neutron_lib.callbacks import registry
 from neutron_lib import context as n_context
 from neutron_lib import exceptions as n_exc
 from oslo_config import cfg
-from oslo_db import exception as os_db_exc
 from oslo_log import log
 
 from networking_ovn.common.constants import OVN_ML2_MECH_DRIVER_NAME
 
+from neutron.objects import ports as port_obj
 from neutron.services.trunk import constants as trunk_consts
 from neutron.services.trunk.drivers import base as trunk_base
 from neutron_lib.api.definitions import portbindings
@@ -41,40 +41,96 @@ class OVNTrunkHandler(object):
     def __init__(self, plugin_driver):
         self.plugin_driver = plugin_driver
 
-    def _set_binding_profile(self, port_id, parent_port, tag=None):
-        context = n_context.get_admin_context()
-        binding_profile = {}
-        if parent_port and tag:
-            binding_profile = {'parent_name': parent_port, 'tag': tag}
-        port = {'port': {'binding:profile': binding_profile}}
-        try:
-            self.plugin_driver._plugin.update_port(context, port_id, port)
-        except (os_db_exc.DBReferenceError, n_exc.PortNotFound):
-            LOG.debug("Port not found trying to set binding_profile: %s",
-                      port_id)
-
     def _set_sub_ports(self, parent_port, subports):
-        for port in subports:
-            self._set_binding_profile(port.port_id, parent_port,
-                                      tag=port.segmentation_id)
+        txn = self.plugin_driver._nb_ovn.transaction
+        context = n_context.get_admin_context()
+        with context.session.begin(subtransactions=True), (
+                txn(check_error=True)) as ovn_txn:
+            for port in subports:
+                self._set_binding_profile(context, port, parent_port, ovn_txn)
 
     def _unset_sub_ports(self, subports):
-        for port in subports:
-            self._set_binding_profile(port.port_id, None)
+        txn = self.plugin_driver._nb_ovn.transaction
+        context = n_context.get_admin_context()
+        with context.session.begin(subtransactions=True), (
+                txn(check_error=True)) as ovn_txn:
+            for port in subports:
+                self._unset_binding_profile(context, port, ovn_txn)
+
+    def _set_binding_profile(self, context, subport, parent_port, ovn_txn):
+        db_port = port_obj.Port.get_object(context, id=subport.port_id)
+        if not db_port:
+            LOG.debug("Port not found while trying to set "
+                      "binding_profile: %s",
+                      subport.port_id)
+            return
+        try:
+            db_port.binding.profile.update({
+                'parent_name': parent_port,
+                'tag': subport.segmentation_id})
+            # host + port_id is primary key
+            port_obj.PortBinding.update_object(
+                context, {'profile': db_port.binding.profile},
+                port_id=subport.port_id,
+                host=db_port.binding.host)
+        except n_exc.ObjectNotFound:
+            LOG.debug("Port not found while trying to set "
+                      "binding_profile: %s", subport.port_id)
+            return
+        ovn_txn.add(self.plugin_driver._nb_ovn.set_lswitch_port(
+                    lport_name=subport.port_id,
+                    parent_name=parent_port,
+                    tag=subport.segmentation_id))
+
+    def _unset_binding_profile(self, context, subport, ovn_txn):
+        db_port = port_obj.Port.get_object(context, id=subport.port_id)
+        if not db_port:
+            LOG.debug("Port not found while trying to unset "
+                      "binding_profile: %s",
+                      subport.port_id)
+            return
+        try:
+            db_port.binding.profile.pop('tag', None)
+            db_port.binding.profile.pop('parent_name', None)
+            # host + port_id is primary key
+            port_obj.PortBinding.update_object(
+                context,
+                {'profile': db_port.binding.profile,
+                 'vif_type': portbindings.VIF_TYPE_UNBOUND,
+                 'vif_details': '',
+                 'host': ''},
+                port_id=subport.port_id,
+                host=db_port.binding.host)
+            port_obj.PortBindingLevel.delete_objects(
+                context, port_id=subport.port_id,
+                host=db_port.binding.host)
+        except n_exc.ObjectNotFound:
+            LOG.debug("Port not found while trying to unset "
+                      "binding_profile: %s", subport.port_id)
+            return
+        ovn_txn.add(self.plugin_driver._nb_ovn.set_lswitch_port(
+                    lport_name=subport.port_id,
+                    parent_name=[],
+                    up=False,
+                    tag=[]))
 
     def trunk_created(self, trunk):
-        self._set_sub_ports(trunk.port_id, trunk.sub_ports)
+        if trunk.sub_ports:
+            self._set_sub_ports(trunk.port_id, trunk.sub_ports)
         trunk.update(status=trunk_consts.ACTIVE_STATUS)
 
     def trunk_deleted(self, trunk):
-        self._unset_sub_ports(trunk.sub_ports)
+        if trunk.sub_ports:
+            self._unset_sub_ports(trunk.sub_ports)
 
     def subports_added(self, trunk, subports):
-        self._set_sub_ports(trunk.port_id, subports)
+        if subports:
+            self._set_sub_ports(trunk.port_id, subports)
         trunk.update(status=trunk_consts.ACTIVE_STATUS)
 
     def subports_deleted(self, trunk, subports):
-        self._unset_sub_ports(subports)
+        if subports:
+            self._unset_sub_ports(subports)
         trunk.update(status=trunk_consts.ACTIVE_STATUS)
 
     def trunk_event(self, resource, event, trunk_plugin, payload):
