@@ -21,6 +21,7 @@ from neutron.tests.unit.extensions import test_extraroute
 from neutron.tests.unit.extensions import test_l3
 from neutron.tests.unit.extensions import test_l3_ext_gw_mode as test_l3_gw
 from neutron_lib.api.definitions import portbindings
+from neutron_lib.api.definitions import provider_net as pnet
 from neutron_lib.callbacks import events
 from neutron_lib.callbacks import resources
 from neutron_lib import constants
@@ -558,7 +559,7 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
     def test_update_router_ext_gw_change_subnet(self, gr, ur, gs,
                                                 grps, gp, mock_get_gw):
         self.l3_inst._ovn.is_col_present.return_value = True
-        mock_get_gw.return_value = mock.sentinel.GwRoute
+        mock_get_gw.return_value = [mock.sentinel.GwRoute]
         router = {'router': {'name': 'router'}}
         fake_old_ext_subnet = {'id': 'old-ext-subnet-id',
                                'ip_version': 4,
@@ -616,7 +617,7 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
     def test_update_router_ext_gw_change_ip_address(self, gr, ur, gs,
                                                     grps, gp, mock_get_gw):
         self.l3_inst._ovn.is_col_present.return_value = True
-        mock_get_gw.return_value = mock.sentinel.GwRoute
+        mock_get_gw.return_value = [mock.sentinel.GwRoute]
         router = {'router': {'name': 'router'}}
         # Old gateway info with same subnet and different ip address
         gr_value = copy.deepcopy(self.fake_router_with_ext_gw)
@@ -746,7 +747,7 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
     @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin.get_router')
     def test_disable_snat(self, gr, ur, gs, grps, gp, mock_get_gw, mock_snats,
                           mock_ext_ips):
-        mock_get_gw.return_value = mock.sentinel.GwRoute
+        mock_get_gw.return_value = [mock.sentinel.GwRoute]
         mock_snats.return_value = [mock.sentinel.NAT]
         mock_ext_ips.return_value = False
         router = {'router': {'name': 'router'}}
@@ -792,7 +793,8 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
     @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin._get_floatingip')
     def test_create_floatingip_distributed(self, gf, gp):
         self.l3_inst._ovn.is_col_present.return_value = True
-        gp.return_value = {'mac_address': '00:01:02:03:04:05'}
+        gp.return_value = {'mac_address': '00:01:02:03:04:05',
+                           'network_id': 'port-network-id'}
         gf.return_value = {'floating_port_id': 'fip-port-id'}
         config.cfg.CONF.set_override(
             'enable_distributed_floating_ip', True, group='ovn')
@@ -847,7 +849,6 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
             {'external_ip': '192.168.0.10', 'logical_ip': '10.0.0.6',
              'type': 'dnat_and_snat', 'uuid': 'uuid1'}]
         self.l3_inst.create_floatingip(self.context, 'floatingip')
-        self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_not_called()
         expected_ext_ids = {
             ovn_const.OVN_FIP_EXT_ID_KEY: self.fake_floating_ip['id'],
             ovn_const.OVN_REV_NUM_EXT_ID_KEY: '1',
@@ -855,8 +856,8 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
                 self.fake_floating_ip['port_id'],
             ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: utils.ovn_name(
                 self.fake_floating_ip['router_id'])}
-        self.l3_inst._ovn.set_nat_rule_in_lrouter.assert_called_once_with(
-            'neutron-router-id', 'uuid1',
+        self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
+            'neutron-router-id',
             type='dnat_and_snat',
             logical_ip='10.0.0.10',
             external_ip='192.168.0.10',
@@ -953,16 +954,24 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
             external_ip='192.168.0.10',
             external_ids=expected_ext_ids)
 
+    @mock.patch('neutron.db.db_base_plugin_v2.NeutronDbPluginV2.get_network')
     @mock.patch('neutron.db.db_base_plugin_v2.NeutronDbPluginV2.get_port')
     @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin._get_floatingip')
     @mock.patch('neutron.db.extraroute_db.ExtraRoute_dbonly_mixin.'
                 'update_floatingip')
-    def test_update_floatingip_associate_distributed(self, uf, gf, gp):
+    def _test_update_floatingip_associate_distributed(self, network_type,
+                                                      uf, gf, gp, gn):
         self.l3_inst._ovn.is_col_present.return_value = True
         self.fake_floating_ip.update({'fixed_port_id': None})
-        gp.return_value = {'mac_address': '00:01:02:03:04:05'}
+        gp.return_value = {'mac_address': '00:01:02:03:04:05',
+                           'network_id': 'port-network-id'}
         gf.return_value = self.fake_floating_ip
         uf.return_value = self.fake_floating_ip_new
+
+        fake_network_vlan = self.fake_network
+        fake_network_vlan[pnet.NETWORK_TYPE] = network_type
+        gn.return_value = fake_network_vlan
+
         config.cfg.CONF.set_override(
             'enable_distributed_floating_ip', True, group='ovn')
         self.l3_inst.update_floatingip(self.context, 'id', 'floatingip')
@@ -975,11 +984,23 @@ class OVNL3RouterPlugin(test_mech_driver.OVNMechanismDriverTestCase):
             ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY: utils.ovn_name(
                 self.fake_floating_ip_new['router_id']),
             ovn_const.OVN_FIP_EXT_MAC_KEY: '00:01:02:03:04:05'}
-        self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
-            'neutron-new-router-id', type='dnat_and_snat',
-            logical_ip='10.10.10.10', external_ip='192.168.0.10',
-            external_mac='00:01:02:03:04:05', logical_port='new-port_id',
-            external_ids=expected_ext_ids)
+        if network_type == constants.TYPE_VLAN:
+            self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
+                'neutron-new-router-id', type='dnat_and_snat',
+                logical_ip='10.10.10.10', external_ip='192.168.0.10',
+                logical_port='new-port_id', external_ids=expected_ext_ids)
+        else:
+            self.l3_inst._ovn.add_nat_rule_in_lrouter.assert_called_once_with(
+                'neutron-new-router-id', type='dnat_and_snat',
+                logical_ip='10.10.10.10', external_ip='192.168.0.10',
+                external_mac='00:01:02:03:04:05', logical_port='new-port_id',
+                external_ids=expected_ext_ids)
+
+    def test_update_floatingip_associate_distributed_flat(self):
+        self._test_update_floatingip_associate_distributed(constants.TYPE_FLAT)
+
+    def test_update_floatingip_associate_distributed_vlan(self):
+        self._test_update_floatingip_associate_distributed(constants.TYPE_VLAN)
 
     @mock.patch('neutron.db.l3_db.L3_NAT_dbonly_mixin._get_floatingip')
     @mock.patch('neutron.db.extraroute_db.ExtraRoute_dbonly_mixin.'
@@ -1190,3 +1211,12 @@ class OVNL3ExtrarouteTests(test_l3_gw.ExtGwModeIntTestCase,
             'neutron-fake_device', ip_prefix='0.0.0.0/0', nexthop='120.0.0.2')
         self.l3_inst._ovn.delete_static_route.assert_called_once_with(
             'neutron-fake_device', ip_prefix='0.0.0.0/0', nexthop='120.0.0.1')
+
+    def test_router_update_gateway_upon_subnet_create_max_ips_ipv6(self):
+        super(OVNL3ExtrarouteTests, self). \
+            test_router_update_gateway_upon_subnet_create_max_ips_ipv6()
+        add_static_route_calls = [
+            mock.call(mock.ANY, ip_prefix='0.0.0.0/0', nexthop='10.0.0.1'),
+            mock.call(mock.ANY, ip_prefix='::/0', nexthop='2001:db8::1')]
+        self.l3_inst._ovn.add_static_route.assert_has_calls(
+            add_static_route_calls, any_order=True)
