@@ -13,6 +13,7 @@
 #
 
 import copy
+import uuid
 
 import mock
 
@@ -400,7 +401,8 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
     @mock.patch.object(impl_idl_ovn, 'get_connection', mock.Mock())
     def test_setting_ovsdb_probe_timeout_default_value(self):
         inst = impl_idl_ovn.OvsdbNbOvnIdl(mock.Mock())
-        inst.idl._session.reconnect.set_probe_interval.assert_called_with(0)
+        inst.idl._session.reconnect.set_probe_interval.assert_called_with(
+            60000)
 
     @mock.patch.object(impl_idl_ovn.OvsdbNbOvnIdl, 'ovsdb_connection', None)
     @mock.patch.object(impl_idl_ovn, 'get_connection', mock.Mock())
@@ -695,6 +697,20 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
         address_sets = self.nb_ovn_idl.get_address_sets()
         self.assertEqual(len(address_sets), 4)
 
+    def test_get_port_group_not_supported(self):
+        self._load_nb_db()
+        # Make sure that PG tables doesn't exist in fake db.
+        self._tables.pop('Port_Group', None)
+        port_group = self.nb_ovn_idl.get_port_group(str(uuid.uuid4()))
+        self.assertIsNone(port_group)
+
+    def test_get_port_groups_not_supported(self):
+        self._load_nb_db()
+        # Make sure that PG tables doesn't exist in fake db.
+        self._tables.pop('Port_Group', None)
+        port_groups = self.nb_ovn_idl.get_port_groups()
+        self.assertEqual({}, port_groups)
+
 
 class TestSBImplIdlOvn(TestDBImplIdlOvn):
 
@@ -735,7 +751,8 @@ class TestSBImplIdlOvn(TestDBImplIdlOvn):
     @mock.patch.object(impl_idl_ovn, 'get_connection', mock.Mock())
     def test_setting_ovsdb_probe_timeout_default_value(self):
         inst = impl_idl_ovn.OvsdbSbOvnIdl(mock.Mock())
-        inst.idl._session.reconnect.set_probe_interval.assert_called_with(0)
+        inst.idl._session.reconnect.set_probe_interval.assert_called_with(
+            60000)
 
     @mock.patch.object(impl_idl_ovn.OvsdbSbOvnIdl, 'ovsdb_connection', None)
     @mock.patch.object(impl_idl_ovn, 'get_connection', mock.Mock())
